# frozen_string_literal: true
require 'pundit'
require 'strings-case'

module NexusCqrs
  # Concern used to provide authorisation abilities to handlers and other classes. Overrides pundit's `authorize` method
  # and creates helpers for the permission_provider
  module Auth
    include Pundit::Authorization

    # Overrides pundit's `authorize` method, allowing the message to be passed
    #
    # @see Pundit#authorize
    # @param [NexusCqrs::BaseMessage] message Message to authorise against
    def authorize(message, record, query = nil, policy_class: nil)
      # Populate the query from the command, or the params if it's being overriden
      query ||= Strings::Case.snakecase(message.demodularised_class_name) + '?'

      # Retreive the policy class object from the type of record we are passing in
      policy_class ||= Pundit::PolicyFinder.new(record).policy

      # Pull context variables from command
      user = message.metadata[:current_user]
      global_permissions = message.metadata[:global_permissions]

      # Raise issue if policy class doesn't exist
      raise Pundit::NotAuthorizedError,
            query: query,
            record: record,
            message: "There is no policy class available for #{record.class}" if policy_class.nil?

      # Instantiate new policy class, with context
      policy = policy_class.new(UserContext.new(user, global_permissions), record)
      raise Pundit::NotAuthorizedError, query: query, record: record, policy: policy unless policy.public_send(query)

      record.is_a?(Array) ? record.last : record
    end

    # Helper method for creating a permissions provider object from a query object. This allows certain permissions
    # to be checked inside the command handler, as opposed to inside the policy
    #
    # @param [NexusCqrs::BaseMessage] message Create the PermissionProvider using the message metadata
    # @return [PermissionProvider] A new instance of the permission provider
    def permission_provider(message)
      PermissionProvider.new(message.metadata[:current_user], message.metadata[:global_permissions])
    end

    def pundit_user
      nil
    end

    def current_user
      return super if defined?(super)

      nil
    end
  end
end
