# frozen_string_literal: true
module NexusCqrs
  module Auth
    # Concern used to integrate models to the permissions system. Including this module to a model will assume the
    # model can be "owned" by a user. When the model is created, permissions will automatically be assigned to the user
    # and permissions can be validated and "repaired" retroactively.
    module Ownable
      extend ActiveSupport::Concern

      included do
        class_attribute :granted_permissions, default: {}

        # Default relationship to <class>_permissions
        class_attribute :permissions_relation, default: top_ancestor_class.name.downcase + "_permissions"
        class_attribute :owner_column, default: :user_id

        before_create :assign_permissions

        define_model_callbacks :create_permissions
      end

      module ClassMethods
        # Gets the class that this module is included in
        def top_ancestor_class
          ancestors.first
        end
      end

      def assign_permissions
        # As we are doing this on before_create, we must "build" this association, as opposed to creating it. This
        # ensures validation is passed before saving this parent Collection.
        granted_permissions.each do |p|
          unless permissions.where(permission: p, entity_id: id, user_id: owner_id).exists?
            permissions.build(user_id: owner_id, permission: p)
          end
        end
      end

      def owner_id
        send(owner_column)
      end

      def permissions
        if respond_to?(permissions_relation)
          return send(permissions_relation)
        end

        raise OwnableRelationshipNotSet, "Permissions relation not set.
  Set it on your model with `self.permissions_relation = :xxx_permissions`, and ensure relationship has been created"
      end
    end

    class OwnableRelationshipNotSet < StandardError
    end
  end
end
