# frozen_string_literal: true
require 'request_store'

module NexusCqrs
  module Auth
    class PermissionProvider
      def initialize(user_id, global_permissions)
        @user_id = user_id
        @global_permissions = parse_permissions_array(global_permissions)
      end

      # Returns true if the current user has the requested permission on the requested entity (if passed), or globally
      #
      # @param [String] permission_key Permission key to check against
      # @param [ApplicationRecord] permission_model Permission model
      # @param [Integer] entity_id ID of the entity
      # @return [Boolean] Returns true if the current user has this permission on this entity
      # @example Check for permission
      #   permissions.has_permission?('collection:publish', CollectionPermissions, collection.id) #=> true
      def has_permission?(permission_key, permission_model = nil, entity_id = nil)
        return false if @user_id.nil?

        return true if @global_permissions.include?(permission_key)

        # check entity-specific permissions
        unless permission_model.nil?

          # get all permissions for this entity. NOTE: This will be cached per-request.
          permissions = cached_permissions(permission_model)

          # if there are no permissions for this entity and user, return false
          return false if permissions[entity_id].nil?

          # if the permission key is in the user's permissions for this entity, return true
          return true if permissions[entity_id].include?(permission_key)
        end

        false
      end

      # Retrieves a list of permissions assigned to a user for a specific entity
      #
      # @param [ApplicationRecord] permission_model Permission model
      # @param [Integer] entity_id ID of the entity
      # @return [Array] Returns an array of hashes representing permission keys, along with their global status
      # @example Get a list of permissions
      #   permissions.for_user_on_entity(CollectionPermissions, collection.id) #=>
      #     [
      #       {:global=>false, :key=>"collection:discard"},
      #       {:global=>false, :key=>"collection:publish"},
      #       {:global=>false, :key=>"collection:view_under_moderation"},
      #       {:global=>false, :key=>"collection:set_status"}
      #     ]
      def for_user_on_entity(permission_model, entity_id)
        return [] if @user_id.nil?

        # retrieve entity-specific permissions from cached user permissions and map to hash
        entity_permissions = if cached_permissions(permission_model).key?(entity_id)
          cached_permissions(permission_model)[entity_id]
            .map { |p| { global: false, key: p } }
        else
          []
        end

        # Map global permissions to hash
        global_permissions = @global_permissions.map { |p| { global: true, key: p } }

        # Combine hashes and ensure global permissions take priority
        (global_permissions + entity_permissions).uniq { |p| p[:key] }
      end

      # Retrieves a list of permissions assigned to a user for ANY entity ID
      #
      # @param [Class] permission_model Permission model class
      # @return [Hash] Returns a hash representing each entity and the user's permissions
      # @example Get a list of permissions
      #   permissions.for_user(CollectionPermissions) #=>
      #     {
      #       1 => ["collection:discard"],
      #       2 => ["collection:discard"],
      #       3 => ["collection:discard", "collection:edit"],
      #       4 => ["collection:discard"],
      #     }
      def for_user(permission_model)
        return {} if @user_id.nil?

        permissions = {}

        # retrieve entity-specific permissions from DB and map to hash
        permission_model.where(user_id: @user_id).each do |p|
          if permissions[p.entity_id].nil?
            permissions[p.entity_id] = [p.permission]
          else
            permissions[p.entity_id] << p.permission
          end
        end

        permissions
      end

      private

      def parse_permissions_array(permissions_array)
        return [] if permissions_array.nil?

        permissions = []

        permissions_array.each do |entity, action_array|
          action_array.each do |action|
            permissions << entity + ":" + action
          end
        end

        permissions
      end

      # Retrieve all the permissions for this user for a specific model and cache it for this request (as all calls to
      # a PermissionProvider in the same request will all have the same permissions, this saves on many SQL calls)
      #
      # @param [Class] permission_model Class for the permission model being used.
      # @see PermissionProvider#for_user
      def cached_permissions(permission_model)
        ::RequestStore.store["permissions_#{permission_model.name}"] ||= for_user(permission_model)
      end
    end
  end
end
