# frozen_string_literal: true
module NexusCqrs
  # All messages passed through the message bus will extend this class. Commands and Queries are both extended types of
  # `BaseMessage`. This is mainly used to store metadata for Commands/Queries (such as the user context), as well
  # as helper methods for retrieving the policy method for this Command/Query/Message
  #
  # @since 0.1.0
  class BaseMessage
    # Sets metadata on this message.
    #
    # @param [Symbol] key Metadata key
    # @param [Object] value Metadata value
    # @example Set the current user on a command/query
    #   command.set_metadata(:current_user, user)
    # @since 0.1.0
    def set_metadata(key, value)
      @metadata = {} unless @metadata
      @metadata[key.to_sym] = value
    end

    # Getter for retrieving the metadata on this message
    #
    # @return [Hash] Any metadata attached to this message
    # @since 0.1.0
    def metadata
      @metadata || {}
    end

    # Helper method for retrieving the name of the class used to define the auth policy for this message
    #
    # @example Get the policy_class on a `Commands::Mods::DeleteMod` command
    #   command.policy_class #=> "DeleteModPolicy"
    #
    # @return [String] Name of the policy class
    # @deprecated This used to be used to authorise policies before they hit the command bus - requiring a new policy
    #   for every message. E.g. `DeleteModPolicy`, `UpdateModPolicy`, `CreateModPolicy` etc. It is now recommended to
    #   simple call `authorise` within the handler as it is far more flexible
    # @since 0.1.0
    def policy_class
      demodularised_class_name + 'Policy'
    end

    # Helper method for retrieving the demodularised name of the class used to define the auth policy for this message
    #
    # @example Get the demodularised_class_name on a `Commands::Mods::DeleteMod` command
    #   command.policy_class #=> "DeleteMod"
    #
    # @return [String] Name of the policy class
    # @since 0.1.0
    def demodularised_class_name
      self.class.name.split('::').last
    end
  end
end
