# frozen_string_literal: true
require 'thread_safe'
require 'middleware'

module NexusCqrs
  # The command bus is responsible for registering and invoking the handlers. It stores a simple list of all registered
  # commands, along with their handlers. When a message is passed to `call`, the middleware is triggered for the message
  # then the handler is invoked.
  #
  # @since 0.1.0
  class CommandBus
    UnregisteredHandler = Class.new(StandardError)
    MultipleHandlers    = Class.new(StandardError)

    def initialize(middleware: nil)
      @handlers = ThreadSafe::Cache.new
      @middleware = middleware || Middleware::Builder.new
    end

    # Registers a handler for a command.
    #
    # @param [NexusCqrs::BaseMessage] message Message to register a handler for
    # @param [NexusCqrs::BaseCommandHandler] handler CommandHandler/QueryHandler to be invoked for this message
    # @raise [MultipleHandlers] If the message already has a handler registered, an error will be raised
    #
    # @since 0.1.0
    def register(message, handler)
      raise MultipleHandlers, "Multiple handlers not allowed for #{message}" if handlers[message]

      handlers[message] = handler
    end

    # Invokes a handler from a message
    #
    # @param [NexusCqrs::BaseMessage] message Message used to determine which handler to invoke
    #
    # @since 0.1.0
    def call(message)
      runner = Middleware::Builder.new
      runner.use(@middleware)
      runner.use(->(message_) { handler_for_command(message_).call(message_) })
      runner.call(message)
    end

    # Return a list of registered handlers
    #
    # @return [ThreadSafe::Cache] ThreadSafe::Cache object containing a list of all the registered handlers
    #
    # @since 0.1.0
    def registered_handlers
      handlers
    end

    # Removes all registered handlers from the bus. This can be useful in tests - as we don't want state to persist
    # across handlers
    #
    # @since 0.1.0
    def clear_handlers
      @handlers = ThreadSafe::Cache.new
    end

    attr_reader :handlers

    private

    # Looks up the handler for a specific command
    #
    # @since 0.1.0
    def handler_for_command(command)
      unregistered_handler = proc { raise UnregisteredHandler, "Missing handler for #{command.class}" }
      handlers.fetch(command.class, &unregistered_handler)
    end
  end
end
