# frozen_string_literal: true

module NexusCqrs
  class CommandExecutor
    # @param [NexusCqrs::CommandBus] command_bus
    def initialize(command_bus)
      @bus = command_bus
      @logger = logger
    end

    # Get logger instance depending on runtime environment.
    def logger
      defined?(Rails) && defined?(Rails.logger) ? Rails.logger : Logger.new(STDOUT)
    end

    def autoregister(base_class, dir = 'app/domain/commands', ignore_strings = ['.rb', 'app/domain/'])
      if defined?(Rails)
        # Iterate over the directory passed and find all ruby files.
        Dir["#{dir}/**/*.rb"].each do |file|
          # Constantize class name to constant to force rails to autoload this class.
          # Note that autoloading won't work when testing this gem standalone, but this does trigger the necessary
          # loading when in a rails environment.
          ignore_strings.each do |i|
            file.slice!(i)
          end
          @logger.debug { "Attempting constantize of #{file}" }
          file.camelize.constantize
        rescue NameError => e
          @logger.warn { "Failed autoregister #{file.camelize}, received NameError: #{e}" }
        end
      end

      ObjectSpace.each_object(Class).select { |klass| klass < base_class }.each do |c|
        @logger.debug { "Attempting auto registration of #{c}" }
        handler_name = (c.to_s + "Handler")
        begin
          handler = handler_name.constantize.new
        rescue NameError
          @logger.warn { "Command `#{c}` tried to autoregister `#{handler_name}` but the class was not found" }
          next
        end

        if handler.respond_to?(:call)
          register_command(c, handler)
        else
          @logger.warn { "Command `#{c}` tried to autoregister `#{handler_name}` but `call` method did not exist" }
        end
      end
    end

    # Returns the bus used by this executor
    #
    # @return [NexusCqrs::CommandBus] command_bus
    def command_bus
      @bus
    end

    # Executes a specific handler on the bus, passing in the provided message (Query/Command)
    #
    # @param [NexusCqrs::BaseMessage] message
    def execute(message)
      @bus.call(message)
    end

    # Helper method for registering a handler on the bus
    #
    # @see CommandBus#register
    # @param [NexusCqrs::BaseMessage] message
    # @param [NexusCqrs::BaseCommandHandler] handler
    def register_command(message, handler)
      @bus.register(message, handler)
    end

    # Clears all handlers from the bus and invokes the block passed to `configure_handlers` to re-register all the
    # handlers again. This can be useful in tests - as we don't want state to persist across handlers
    def reregister_handlers
      return if @register_handlers.nil?

      @bus.clear_handlers
      @register_handlers.call(self)
    end

    # Configuration method for allowing handlers to be registered in bulk
    #
    # @see CommandBus#register
    # @param [Proc] block Block to execute to register handlers
    # @example Registering all queries
    #   executor.configure_handlers do |executor|
    #     executor.autoregister(NexusCqrs::BaseQuery, 'app/domain/queries')
    #   end
    # @example Registering a query manually
    #   executor.configure_handlers do |executor|
    #     executor.register_command(NexusCqrs::BaseQuery, NexusCqrs::BaseQueryHandler)
    #   end
    def configure_handlers(&block)
      @register_handlers = block
      @register_handlers.call(self)
    end
  end
end
