require_relative '../../../lib/nexus_cqrs'

class EntityPermission

  def initialize(entity_id:, permission:)
    self.entity_id = entity_id
    self.permission = permission
  end

  attr_accessor :entity_id
  attr_accessor :permission
end

class OtherEntityPermission

  def initialize(entity_id:, permission:)
    self.entity_id = entity_id
    self.permission = permission
  end

  attr_accessor :entity_id
  attr_accessor :permission
end

describe NexusCqrs::Auth::PermissionProvider do

  subject { NexusCqrs::Auth::PermissionProvider.new(1, {'global' => ['test']}) }

  before do
    RequestStore.clear!

    allow(entity_permission).to receive(:name).and_return("EntityPermission")
    allow(other_entity_permission).to receive(:name).and_return("OtherEntityPermission")
  end

  let(:entity_permission) { double(:EntityPermission) }
  let(:other_entity_permission) { double(:EntityPermission) }

  it "retreives permissions for user in correct format" do

    expect(entity_permission).to receive(:where).and_return([
                                                  EntityPermission.new(entity_id: 1, permission: 'test:permission'),
                                                  EntityPermission.new(entity_id: 2, permission: 'test:permission')
                                                ])

    expect(subject.for_user(entity_permission)).to eq(
                                         {
                                           1 => ['test:permission'],
                                           2 => ['test:permission']
                                         }
                                       )
  end

  it "retreives permissions for user on a specific entity (including global permissions) in correct format" do

    expect(entity_permission).to receive(:where).with(user_id: 1).and_return([
                                                   EntityPermission.new(entity_id: 1, permission: 'test:permission'),
                                                   EntityPermission.new(entity_id: 2, permission: 'test:permission')
                                                 ])

    # Ensure the individual queries to get user+entity permissions is not ran.
    expect(entity_permission).to_not receive(:where).with(entity_id: 1, user_id: 1)
    expect(entity_permission).to_not receive(:where).with(entity_id: 3, user_id: 1)

    expect(subject.for_user_on_entity(entity_permission, 1)).to eq(
                                                      [{:global => true, :key => "global:test"},
                                                       {:global => false, :key => "test:permission"}]
                                                    )

    expect(subject.for_user_on_entity(entity_permission, 3)).to eq(
                                                      [{:global => true, :key => "global:test"}]
                                                    )
  end

  it "correctly caches permissions across entity models" do

    allow(entity_permission).to receive(:name).and_return("Entity1Permission")
    allow(other_entity_permission).to receive(:name).and_return("Entity2Permission")

    expect(entity_permission).to receive(:where).with(user_id: 1).and_return([
                                                                   EntityPermission.new(entity_id: 1, permission: 'test:permission'),
                                                                   EntityPermission.new(entity_id: 2, permission: 'test:permission')
                                                                 ])

    expect(other_entity_permission).to receive(:where).with(user_id: 1).and_return([
                                                                   OtherEntityPermission.new(entity_id: 3, permission: 'test:permission'),
                                                                   OtherEntityPermission.new(entity_id: 4, permission: 'test:permission')
                                                                  ])

    # Check two different models are being cached correctly
    expect(subject.for_user_on_entity(entity_permission, 1)).to eq(
                                                      [{:global => true, :key => "global:test"},
                                                       {:global => false, :key => "test:permission"}]
                                                    )

    expect(subject.for_user_on_entity(other_entity_permission, 4)).to eq(
                                                       [{:global => true, :key => "global:test"},
                                                        {:global => false, :key => "test:permission"}]
                                                     )


    # Check state hasn't been left from previous model call
    expect(subject.for_user_on_entity(entity_permission, 3)).to eq(
                                                       [{:global => true, :key => "global:test"}]
                                                     )

    expect(subject.for_user_on_entity(other_entity_permission, 1)).to eq(
                                                       [{:global => true, :key => "global:test"}]
                                                     )
  end
end