# frozen_string_literal: true
require_relative '../../lib/nexus_cqrs'

class TestCommand < NexusCqrs::BaseCommand
  attr_reader :property
  def initialize(property = nil)
    @property = property
  end
end

class TestCommandHandler < NexusCqrs::BaseCommandHandler
end

class TestQuery < NexusCqrs::BaseQuery
end

class TestQueryHandler < NexusCqrs::BaseQueryHandler
end

class TestMiddleware < NexusCqrs::BaseMiddleware
  def initialize(next_, index)
    super(next_)
    @index = index
  end

  def call(command)
    before_handlers = command.metadata[:before_handlers] || []
    before_handlers << @index
    command.set_metadata('before_handlers', before_handlers)
    @next.call(command)
    after_handlers = command.metadata[:after_handlers] || []
    after_handlers << @index
    command.set_metadata('after_handlers', after_handlers)
    command
  end
end

describe NexusCqrs::CommandBus do
  let(:command_bus) { NexusCqrs::CommandBus.new }
  let(:test_command_handler) { TestCommandHandler.new }
  let(:test_query_handler) { TestQueryHandler.new }

  it "allows a command/query to be registered" do
    expect(command_bus.registered_handlers.size).to(be(0))

    command_bus.register(TestCommand, test_command_handler)

    expect(command_bus.registered_handlers.size).to(be(1))

    command_bus.register(TestQuery, test_query_handler)

    expect(command_bus.registered_handlers.size).to(be(2))
  end

  it "calls the correct handler" do
    test_command = TestCommand.new
    dbl = double("TestCommandHandler")
    expect(dbl).to(receive(:call))

    command_bus.register(TestCommand, dbl)
    command_bus.call(test_command)
  end

  it "calls middleware in the correct order" do
    test_command = TestCommand.new('test')
    stack = Middleware::Builder.new do |b|
      b.use(TestMiddleware, '1')
      b.use(TestMiddleware, '2')
    end
    bus = NexusCqrs::CommandBus.new(middleware: stack)
    bus.register(TestCommand, ->(command) do
      # HANDLER
      command
    end)

    result = bus.call(test_command)

    expect(result.property).to(eq('test'))
    expect(result.metadata[:before_handlers]).to(eq(%w[1 2]))
    expect(result.metadata[:after_handlers]).to(eq(%w[2 1]))
  end
end
